@@ -42,7 +42,7 @@
  *
  * You should provide a #ClutterTimeline and bind it to the #ClutterAlpha
  * instance using clutter_alpha_set_timeline(). You should also set an
- * "animation mode", either by using the #ClutterAnimatioMode values that
+ * "animation mode", either by using the #ClutterAnimationMode values that
  * Clutter itself provides or by registering custom functions using
  * clutter_alpha_register_func().
  *
@@ -565,7 +565,7 @@ clutter_alpha_new_with_func (ClutterTimeline  *timeline,
  * clutter_alpha_get_mode:
  * @alpha: a #ClutterAlpha
  *
- * Retrieves the #ClutterAnimatioMode used by @alpha.
+ * Retrieves the #ClutterAnimationMode used by @alpha.
  *
  * Return value: the animation mode
  *
@@ -603,10 +603,9 @@ clutter_ease_out_quad (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble p = clutter_timeline_get_progress (timeline);
 
-  return -1.0 * (t /= d) * (t - 2);
+  return -1.0 * p * (p - 2);
 }
 
 static gdouble
@@ -614,13 +613,16 @@ clutter_ease_in_out_quad (ClutterAlpha *alpha,
                           gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / (d / 2);
 
-  if ((t /= d / 2) < 1)
-    return 0.5 * t * t;
+  if (p < 1)
+    return 0.5 * p * p;
 
-  return -0.5 * ((--t) * (t - 2) - 1);
+  p -= 1;
+
+  return -0.5 * (p * (p - 2) - 1);
 }
 
 static gdouble
@@ -628,10 +630,9 @@ clutter_ease_in_cubic (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble p = clutter_timeline_get_progress (timeline);
 
-  return (t /= d) * t * t;
+  return p * p * p;
 }
 
 static gdouble
@@ -639,10 +640,11 @@ clutter_ease_out_cubic (ClutterAlpha *alpha,
                         gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / d - 1;
 
-  return (t = t / d - 1) * t * t + 1;
+  return p * p * p + 1;
 }
 
 static gdouble
@@ -650,13 +652,16 @@ clutter_ease_in_out_cubic (ClutterAlpha *alpha,
                            gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / (d / 2);
 
-  if ((t /= d / 2) < 1)
-    return 0.5 * t * t * t;
+  if (p < 1)
+    return 0.5 * p * p * p;
 
-  return 0.5 * ((t -= 2) * t * t + 2);
+  p -= 2;
+
+  return 0.5 * (p * p * p + 2);
 }
 
 static gdouble
@@ -664,10 +669,9 @@ clutter_ease_in_quart (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble p = clutter_timeline_get_progress (timeline);
 
-  return (t /= d) * t * t * t;
+  return p * p * p * p;
 }
 
 static gdouble
@@ -675,10 +679,11 @@ clutter_ease_out_quart (ClutterAlpha *alpha,
                         gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / d - 1;
 
-  return -0.5 * ((t = t / d - 1) * t * t * t - 1);
+  return -1.0 * (p * p * p * p - 1);
 }
 
 static gdouble
@@ -686,13 +691,16 @@ clutter_ease_in_out_quart (ClutterAlpha *alpha,
                            gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / (d / 2);
 
-  if ((t /= d / 2) < 1)
-    return 0.5 * t * t * t * t;
+  if (p < 1)
+    return 0.5 * p * p * p * p;
 
-  return -0.5 * ((t -= 2) * t * t * t - 2);
+  p -= 2;
+
+  return -0.5 * (p * p * p * p - 2);
 }
 
 static gdouble
@@ -700,10 +708,9 @@ clutter_ease_in_quint (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
  {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble p = clutter_timeline_get_progress (timeline);
 
-  return (t /= d) * t * t * t * t;
+  return p * p * p * p * p;
 }
 
 static gdouble
@@ -711,10 +718,11 @@ clutter_ease_out_quint (ClutterAlpha *alpha,
                         gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / d - 1;
 
-  return (t = t / d - 1) * t * t * t * t + 1;
+  return p * p * p * p * p + 1;
 }
 
 static gdouble
@@ -722,13 +730,16 @@ clutter_ease_in_out_quint (ClutterAlpha *alpha,
                            gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / (d / 2);
 
-  if ((t /= d / 2) < 1)
-    return 0.5 * t * t * t * t * t;
+  if (p < 1)
+    return 0.5 * p * p * p * p * p;
 
-  return 0.5 * ((t -= 2) * t * t * t * t + 2);
+  p -= 2;
+
+  return 0.5 * (p * p * p * p * p + 2);
 }
 
 static gdouble
@@ -736,8 +747,8 @@ clutter_ease_in_sine (ClutterAlpha *alpha,
                       gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return -1.0 * cos (t / d * G_PI_2) + 1.0;
 }
@@ -747,8 +758,8 @@ clutter_ease_out_sine (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return sin (t / d * G_PI_2);
 }
@@ -758,8 +769,8 @@ clutter_ease_in_out_sine (ClutterAlpha *alpha,
                           gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return -0.5 * (cos (G_PI * t / d) - 1);
 }
@@ -769,8 +780,8 @@ clutter_ease_in_expo (ClutterAlpha *alpha,
                       gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return (t == 0) ? 0.0 : pow (2, 10 * (t / d - 1));
 }
@@ -780,8 +791,8 @@ clutter_ease_out_expo (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return (t == d) ? 1.0 : -pow (2, -10 * t / d) + 1;
 }
@@ -791,8 +802,9 @@ clutter_ease_in_out_expo (ClutterAlpha *alpha,
                           gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p;
 
   if (t == 0)
     return 0.0;
@@ -800,10 +812,14 @@ clutter_ease_in_out_expo (ClutterAlpha *alpha,
   if (t == d)
     return 1.0;
 
-  if ((t /= d / 2) < 1)
-    return 0.5 * pow (2, 10 * (t - 1));
+  p = t / (d / 2);
 
-  return 0.5 * (-pow (2, -10 * --t) + 2);
+  if (p < 1)
+    return 0.5 * pow (2, 10 * (p - 1));
+
+  p -= 1;
+
+  return 0.5 * (-pow (2, -10 * p) + 2);
 }
 
 static gdouble
@@ -811,10 +827,9 @@ clutter_ease_in_circ (ClutterAlpha *alpha,
                       gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble p = clutter_timeline_get_progress (timeline);
 
-  return -1.0 * (sqrt (1 - (t /= d) * t) - 1);
+  return -1.0 * (sqrt (1 - p * p) - 1);
 }
 
 static gdouble
@@ -822,10 +837,11 @@ clutter_ease_out_circ (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / d - 1;
 
-  return sqrt (1 - (t = t / d - 1) * t);
+  return sqrt (1 - p * p);
 }
 
 static gdouble
@@ -833,13 +849,16 @@ clutter_ease_in_out_circ (ClutterAlpha *alpha,
                           gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / (d / 2);
 
-  if ((t /= d / 2) < 1)
-    return -0.5 * (sqrt (1 - t * t) - 1);
+  if (p < 1)
+    return -0.5 * (sqrt (1 - p * p) - 1);
 
-  return 0.5 * (sqrt (1 - (t -= 2) * t) + 1);
+  p -= 2;
+
+  return 0.5 * (sqrt (1 - p * p) + 1);
 }
 
 static gdouble
@@ -847,15 +866,18 @@ clutter_ease_in_elastic (ClutterAlpha *alpha,
                          gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
   gdouble p = d * .3;
   gdouble s = p / 4;
+  gdouble q = t / d;
 
-  if ((t /= d) == 1)
+  if (q == 1)
     return 1.0;
 
-  return -(pow (2, 10 * (t -= 1)) * sin ((t * d - s) * (2 * G_PI) / p));
+  q -= 1;
+
+  return -(pow (2, 10 * q) * sin ((q * d - s) * (2 * G_PI) / p));
 }
 
 static gdouble
@@ -863,15 +885,16 @@ clutter_ease_out_elastic (ClutterAlpha *alpha,
                           gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
   gdouble p = d * .3;
   gdouble s = p / 4;
+  gdouble q = t / d;
 
-  if ((t /= d) == 1)
+  if (q == 1)
     return 1.0;
 
-  return pow (2, -10 * t) * sin ((t * d - s) * (2 * G_PI) / p) + 1.0;
+  return pow (2, -10 * q) * sin ((q * d - s) * (2 * G_PI) / p) + 1.0;
 }
 
 static gdouble
@@ -879,20 +902,27 @@ clutter_ease_in_out_elastic (ClutterAlpha *alpha,
                              gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
   gdouble p = d * (.3 * 1.5);
   gdouble s = p / 4;
+  gdouble q = t / (d / 2);
 
-  if ((t /= d / 2) == 2)
+  if (q == 2)
     return 1.0;
 
-  if (t < 1)
-    return -.5 * (pow (2, 10 * (t -= 1)) * sin ((t * d - s) * (2 * G_PI) / p));
+  if (q < 1)
+    {
+      q -= 1;
+
+      return -.5 * (pow (2, 10 * q) * sin ((q * d - s) * (2 * G_PI) / p));
+    }
   else
     {
-      return pow (2, -10 * (t -= 1))
-           * sin ((t * d - s) * (2 * G_PI) / p)
+      q -= 1;
+
+      return pow (2, -10 * q)
+           * sin ((q * d - s) * (2 * G_PI) / p)
            * .5 + 1.0;
     }
 }
@@ -902,10 +932,9 @@ clutter_ease_in_back (ClutterAlpha *alpha,
                       gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble p = clutter_timeline_get_progress (timeline);
 
-  return (t /= d) * t * ((1.70158 + 1) * t - 1.70158);
+  return p * p * ((1.70158 + 1) * p - 1.70158);
 }
 
 static gdouble
@@ -913,10 +942,11 @@ clutter_ease_out_back (ClutterAlpha *alpha,
                        gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / d - 1;
 
-  return (t = t / d - 1) * t * ((1.70158 + 1) * t + 1.70158) + 1;
+  return p * p * ((1.70158 + 1) * p + 1.70158) + 1;
 }
 
 static gdouble
@@ -924,28 +954,45 @@ clutter_ease_in_out_back (ClutterAlpha *alpha,
                           gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
-  gdouble s = 1.70158;
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
+  gdouble p = t / (d / 2);
+  gdouble s = 1.70158 * 1.525;
 
-  if ((t /= d / 2) < 1)
-    return 0.5 * (t * t * (((s *= (1.525)) + 1) * t - s));
+  if (p < 1)
+    return 0.5 * (p * p * ((s + 1) * p - s));
 
-  return 0.5 * ((t -= 2) * t * (((s *= (1.525)) + 1) * t + s) + 2);
+  p -= 2;
+
+  return 0.5 * (p * p * ((s + 1) * p + s) + 2);
 }
 
 static gdouble
 ease_out_bounce_internal (gdouble t,
                           gdouble d)
 {
-  if ((t /= d) < (1 / 2.75))
-    return 7.5625 * t * t;
-  else if (t < (2 / 2.75))
-    return 7.5625 * (t -= (1.5 / 2.75)) * t + .75;
-  else if (t < (2.5 / 2.75))
-    return 7.5625 * (t -= (2.25 / 2.75)) * t + .9375;
+  gdouble p = t / d;
+
+  if (p < (1 / 2.75))
+    return 7.5625 * p * p;
+  else if (p < (2 / 2.75))
+    {
+      p -= (1.5 / 2.75);
+
+      return 7.5625 * p * p + .75;
+    }
+  else if (p < (2.5 / 2.75))
+    {
+      p -= (2.25 / 2.75);
+
+      return 7.5625 * p * p + .9375;
+    }
   else
-    return 7.5625 * (t -= (2.625 / 2.75)) * t + .984375;
+    {
+      p -= (2.625 / 2.75);
+
+      return 7.5625 * p * p + .984375;
+    }
 }
 
 static gdouble
@@ -960,8 +1007,8 @@ clutter_ease_in_bounce (ClutterAlpha *alpha,
                         gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return ease_in_bounce_internal (t, d);
 }
@@ -971,8 +1018,8 @@ clutter_ease_out_bounce (ClutterAlpha *alpha,
                          gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   return ease_out_bounce_internal (t, d);
 }
@@ -982,8 +1029,8 @@ clutter_ease_in_out_bounce (ClutterAlpha *alpha,
                             gpointer      dummy G_GNUC_UNUSED)
 {
   ClutterTimeline *timeline = alpha->priv->timeline;
-  gdouble t = clutter_timeline_get_current_frame (timeline);
-  gdouble d = clutter_timeline_get_n_frames (timeline);
+  gdouble t = clutter_timeline_get_elapsed_time (timeline);
+  gdouble d = clutter_timeline_get_duration (timeline);
 
   if (t < d / 2)
     return ease_in_bounce_internal (t * 2, d) * 0.5;
