@@ -94,7 +94,8 @@ static const GDebugKey clutter_debug_keys[] = {
   { "shader", CLUTTER_DEBUG_SHADER },
   { "multistage", CLUTTER_DEBUG_MULTISTAGE },
   { "animation", CLUTTER_DEBUG_ANIMATION },
-  { "layout", CLUTTER_DEBUG_LAYOUT }
+  { "layout", CLUTTER_DEBUG_LAYOUT },
+  { "nop-picking", CLUTTER_DEBUG_NOP_PICKING }
 };
 #endif /* CLUTTER_ENABLE_DEBUG */
 
@@ -152,12 +153,15 @@ _clutter_stage_maybe_relayout (ClutterActor *stage)
 void
 _clutter_stage_maybe_setup_viewport (ClutterStage *stage)
 {
-  if (CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_SYNC_MATRICES)
+  if ((CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_SYNC_MATRICES) &&
+      !(CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_STAGE_IN_RESIZE))
     {
       ClutterPerspective perspective;
       gfloat width, height;
 
-      clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
+      clutter_actor_get_preferred_size (CLUTTER_ACTOR (stage),
+                                        NULL, NULL,
+                                        &width, &height);
       clutter_stage_get_perspective (stage, &perspective);
 
       CLUTTER_NOTE (PAINT,
@@ -369,12 +373,15 @@ _clutter_do_pick (ClutterStage   *stage,
 		  ClutterPickMode mode)
 {
   ClutterMainContext *context;
-  guchar              pixel[4];
+  guchar              pixel[4] = { 0xff, 0xff, 0xff, 0xff };
   GLint               viewport[4];
   CoglColor           white;
   guint32             id;
   GLboolean           dither_was_on;
 
+  if (clutter_debug_flags & CLUTTER_DEBUG_NOP_PICKING)
+    return CLUTTER_ACTOR (stage);
+
   context = _clutter_context_get_default ();
 
   _clutter_backend_ensure_context (context->backend, stage);
@@ -405,6 +412,9 @@ _clutter_do_pick (ClutterStage   *stage,
   /* Calls should work under both GL and GLES, note GLES needs RGBA */
   glGetIntegerv(GL_VIEWPORT, viewport);
 
+  /* Make sure Cogl flushes any batched geometry to the GPU driver */
+  cogl_flush ();
+
   /* Read the color of the screen co-ords pixel */
   glReadPixels (x, viewport[3] - y -1, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
@@ -1494,16 +1504,17 @@ clutter_get_option_group_without_init (void)
 /**
  * clutter_init_with_args:
  * @argc: (inout): a pointer to the number of command line arguments
- * @argv: (array length=argc) (inout): a pointer to the array of command line arguments
- * @parameter_string: a string which is displayed in the
+ * @argv: (array length=argc) (inout) (allow-none): a pointer to the array
+ *   of command line arguments
+ * @parameter_string: (allow-none): a string which is displayed in the
  *   first line of <option>--help</option> output, after
  *   <literal><replaceable>programname</replaceable> [OPTION...]</literal>
- * @entries: a %NULL terminated array of #GOptionEntry<!-- -->s
+ * @entries: (allow-none): a %NULL terminated array of #GOptionEntry<!-- -->s
  *   describing the options of your program
- * @translation_domain: a translation domain to use for translating
- *   the <option>--help</option> output for the options in @entries
- *   with gettext(), or %NULL
- * @error: a return location for a #GError
+ * @translation_domain: (allow-none): a translation domain to use for
+ *   translating the <option>--help</option> output for the options in
+ *   @entries with gettext(), or %NULL
+ * @error: (allow-none): a return location for a #GError
  *
  * This function does the same work as clutter_init(). Additionally,
  * it allows you to add your own command line options, and it
@@ -1621,7 +1632,8 @@ clutter_parse_args (int    *argc,
 /**
  * clutter_init:
  * @argc: (inout): The number of arguments in @argv
- * @argv: (array length=argc) (inout): A pointer to an array of arguments.
+ * @argv: (array length=argc) (inout) (allow-none): A pointer to an array
+ *   of arguments.
  *
  * It will initialise everything needed to operate with Clutter and
  * parses some standard command line options. @argc and @argv are
