@@ -48,6 +48,10 @@ typedef CoglFuncPtr (*GLXGetProcAddressProc) (const guint8 *procName);
 #include "cogl-gles2-wrapper.h"
 #endif
 
+#ifdef HAVE_COGL_GL
+#define glClientActiveTexture ctx->pf_glClientActiveTexture
+#endif
+
 #ifdef COGL_GL_DEBUG
 /* GL error to string conversion */
 static const struct {
@@ -98,7 +102,7 @@ cogl_clear (const CoglColor *color, gulong buffers)
       GE( glClearColor (cogl_color_get_red_float (color),
 			cogl_color_get_green_float (color),
 			cogl_color_get_blue_float (color),
-			0.0) );
+			cogl_color_get_alpha_float (color)) );
       gl_buffers |= GL_COLOR_BUFFER_BIT;
     }
 
@@ -216,6 +220,9 @@ cogl_get_enable ()
 void
 cogl_set_depth_test_enabled (gboolean setting)
 {
+  /* Currently the journal can't track changes to depth state... */
+  _cogl_journal_flush ();
+
   if (setting)
     {
       glEnable (GL_DEPTH_TEST);
@@ -236,6 +243,9 @@ cogl_set_backface_culling_enabled (gboolean setting)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  /* Currently the journal can't track changes to backface culling state... */
+  _cogl_journal_flush ();
+
   ctx->enable_backface_culling = setting;
 }
 
@@ -318,7 +328,7 @@ set_clip_plane (GLint plane_num,
   _cogl_current_matrix_rotate (angle, 0.0f, 0.0f, 1.0f);
   _cogl_current_matrix_translate (-vertex_a[0], -vertex_a[1], -vertex_a[2]);
 
-  _cogl_current_matrix_state_flush ();
+  _cogl_flush_matrix_stacks ();
 
   plane[0] = 0;
   plane[1] = -1.0;
@@ -388,9 +398,15 @@ _cogl_add_stencil_clip (float x_offset,
 			float height,
 			gboolean first)
 {
+  CoglHandle current_source;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  _cogl_material_flush_gl_state (ctx->stencil_material, NULL);
+  _cogl_journal_flush ();
+
+  /* temporarily swap in our special stenciling material */
+  current_source = cogl_handle_ref (ctx->source_material);
+  cogl_set_source (ctx->stencil_material);
 
   if (first)
     {
@@ -443,9 +459,17 @@ _cogl_add_stencil_clip (float x_offset,
       _cogl_set_current_matrix (COGL_MATRIX_MODELVIEW);
     }
 
+  /* make sure our rectangles hit the stencil buffer before we restore
+   * the stencil function / operation */
+  _cogl_journal_flush ();
+
   /* Restore the stencil mode */
   GE( glStencilFunc (GL_EQUAL, 0x1, 0x1) );
   GE( glStencilOp (GL_KEEP, GL_KEEP, GL_KEEP) );
+
+  /* restore the original source material */
+  cogl_set_source (current_source);
+  cogl_handle_unref (current_source);
 }
 
 void
@@ -472,11 +496,19 @@ _cogl_disable_clip_planes (void)
   GE( glDisable (GL_CLIP_PLANE0) );
 }
 
+/* XXX: This should be deprecated and Cogl should be left to manage
+ * the glViewport automatically when switching draw buffers. */
 void
 cogl_viewport (guint width,
 	       guint height)
 {
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  COGL_NOTE (MISC, "glViewport(0, 0, %u, %u)", width, height);
   GE( glViewport (0, 0, width, height) );
+
+  ctx->viewport_width = width;
+  ctx->viewport_height = height;
 }
 
 void
@@ -490,7 +522,7 @@ _cogl_setup_viewport (guint width,
   float z_camera;
   CoglMatrix projection_matrix;
 
-  GE( glViewport (0, 0, width, height) );
+  cogl_viewport (width, height);
 
   /* For Ortho projection.
    * _cogl_current_matrix_ortho (0, width, 0,  height, -1, 1);
@@ -540,6 +572,7 @@ _cogl_setup_viewport (guint width,
   cogl_get_projection_matrix (&projection_matrix);
   z_camera = 0.5 * projection_matrix.xx;
 
+  _cogl_set_current_matrix (COGL_MATRIX_MODELVIEW);
   _cogl_current_matrix_identity ();
   _cogl_current_matrix_translate (-0.5f, -0.5f, -z_camera);
   _cogl_current_matrix_scale (1.0f / width, -1.0f / height, 1.0f / width);
@@ -554,6 +587,9 @@ cogl_get_features (void)
   if (!ctx->features_cached)
     _cogl_features_init ();
 
+  if (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS)
+    ctx->feature_flags &= ~COGL_FEATURE_VBOS;
+
   return ctx->feature_flags;
 }
 
@@ -568,21 +604,18 @@ cogl_features_available (CoglFeatureFlags features)
   return (ctx->feature_flags & features) == features;
 }
 
+/* XXX: This function should be deprecated, and replaced with a
+ * cogl_draw_buffer_get_size() API instead. We don't support offset
+ * viewports, and you can't have floating point viewport sizes. */
 void
 cogl_get_viewport (float v[4])
 {
-  /* FIXME: cogl_get_viewport should return a gint vec */
-#if defined (HAVE_COGL_GLES2) || defined (HAVE_COGL_GLES)
-  GLint viewport[4];
-  int i;
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  glGetIntegerv (GL_VIEWPORT, viewport);
-
-  for (i = 0; i < 4; i++)
-    v[i] = (float)(viewport[i]);
-#else
-  glGetFloatv (GL_VIEWPORT, v);
-#endif
+  v[0] = 0;
+  v[1] = 0;
+  v[2] = ctx->viewport_width;
+  v[3] = ctx->viewport_height;
 }
 
 void
@@ -628,6 +661,9 @@ cogl_set_fog (const CoglColor *fog_color,
   GLfloat fogColor[4];
   GLenum gl_mode = GL_LINEAR;
 
+  /* The cogl journal doesn't currently track fog state changes */
+  _cogl_journal_flush ();
+
   fogColor[0] = cogl_color_get_red_float (fog_color);
   fogColor[1] = cogl_color_get_green_float (fog_color);
   fogColor[2] = cogl_color_get_blue_float (fog_color);
@@ -665,6 +701,9 @@ cogl_set_fog (const CoglColor *fog_color,
 void
 cogl_disable_fog (void)
 {
+  /* Currently the journal can't track changes to fog state... */
+  _cogl_journal_flush ();
+
   glDisable (GL_FOG);
 }
 
@@ -672,7 +711,148 @@ cogl_disable_fog (void)
 void
 cogl_flush_gl_state (int flags)
 {
-  _cogl_current_matrix_state_flush ();
+  _cogl_flush_matrix_stacks ();
 }
 #endif
 
+void
+cogl_flush (void)
+{
+  _cogl_journal_flush ();
+}
+
+void
+cogl_read_pixels (int x,
+                  int y,
+                  int width,
+                  int height,
+                  CoglReadPixelsFlags source,
+                  CoglPixelFormat format,
+                  guint8 *pixels)
+{
+  GLint   viewport[4];
+  GLint   viewport_height;
+  int     rowstride = width * 4;
+  guint8  *temprow;
+
+  g_return_if_fail (format == COGL_PIXEL_FORMAT_RGBA_8888);
+  g_return_if_fail (source == COGL_READ_PIXELS_COLOR_BUFFER);
+
+  temprow = g_alloca (rowstride * sizeof (guint8));
+
+  glGetIntegerv (GL_VIEWPORT, viewport);
+  viewport_height = viewport[3];
+
+  /* The y co-ordinate should be given in OpenGL's coordinate system
+     so 0 is the bottom row */
+  y = viewport_height - y - height;
+
+  /* Setup the pixel store parameters that may have been changed by
+     Cogl */
+  glPixelStorei (GL_PACK_ALIGNMENT, 4);
+#ifdef HAVE_COGL_GL
+  glPixelStorei (GL_PACK_ROW_LENGTH, 0);
+  glPixelStorei (GL_PACK_SKIP_PIXELS, 0);
+  glPixelStorei (GL_PACK_SKIP_ROWS, 0);
+#endif /* HAVE_COGL_GL */
+
+  /* make sure any batched primitives get emitted to the GL driver before
+   * issuing our read pixels... */
+  cogl_flush ();
+
+  glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+
+  /* TODO: consider using the GL_MESA_pack_invert extension in the future
+   * to avoid this flip... */
+
+  /* vertically flip the buffer in-place */
+  for (y = 0; y < height / 2; y++)
+    {
+      if (y != height - y - 1) /* skip center row */
+        {
+          memcpy (temprow,
+                  pixels + y * rowstride, rowstride);
+          memcpy (pixels + y * rowstride,
+                  pixels + (height - y - 1) * rowstride, rowstride);
+          memcpy (pixels + (height - y - 1) * rowstride,
+                  temprow,
+                  rowstride);
+        }
+    }
+}
+
+void
+cogl_begin_gl (void)
+{
+  CoglMaterialFlushOptions options;
+  gulong enable_flags;
+  int i;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (ctx->in_begin_gl_block)
+    {
+      static gboolean shown = FALSE;
+      if (!shown)
+        g_warning ("You should not nest cogl_begin_gl/cogl_end_gl blocks");
+      shown = TRUE;
+      return;
+    }
+  ctx->in_begin_gl_block = TRUE;
+
+  /* Flush all batched primitives */
+  cogl_flush ();
+
+  /* Flush our clipping state to GL */
+  cogl_clip_ensure ();
+
+  /* Flush any client side matrix state */
+  _cogl_flush_matrix_stacks ();
+
+
+  /* Setup the state for the current material */
+
+  /* We considered flushing a specific, minimal material here to try and
+   * simplify the GL state, but decided to avoid special cases and second
+   * guessing what would be actually helpful.
+   *
+   * A user should instead call cogl_set_source_color4ub() before
+   * cogl_begin_gl() to simplify the state flushed.
+   */
+  options.flags = 0;
+  _cogl_material_flush_gl_state (ctx->source_material, &options);
+
+  /* FIXME: This api is a bit yukky, ideally it will be removed if we
+   * re-work the cogl_enable mechanism */
+  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+
+  if (ctx->enable_backface_culling)
+    enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
+
+  cogl_enable (enable_flags);
+
+  /* Disable all client texture coordinate arrays */
+  for (i = 0; i < ctx->n_texcoord_arrays_enabled; i++)
+    {
+      GE (glClientActiveTexture (GL_TEXTURE0 + i));
+      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
+    }
+  ctx->n_texcoord_arrays_enabled = 0;
+}
+
+void
+cogl_end_gl (void)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (!ctx->in_begin_gl_block)
+    {
+      static gboolean shown = FALSE;
+      if (!shown)
+        g_warning ("cogl_end_gl is being called before cogl_begin_gl");
+      shown = TRUE;
+      return;
+    }
+  ctx->in_begin_gl_block = FALSE;
+}
+
