@@ -76,17 +76,17 @@ _cogl_path_stroke_nodes ()
 {
   guint   path_start = 0;
   gulong  enable_flags = COGL_ENABLE_VERTEX_ARRAY;
+  CoglMaterialFlushOptions options;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
   cogl_enable (enable_flags);
 
-  _cogl_material_flush_gl_state (ctx->source_material,
-                                 COGL_MATERIAL_FLUSH_DISABLE_MASK,
-                                 (guint32)~0, /* disable all texture layers */
-                                 NULL);
-  _cogl_current_matrix_state_flush ();
+  options.flags = COGL_MATERIAL_FLUSH_DISABLE_MASK;
+  options.disable_layers = (guint32)~0;
+  _cogl_material_flush_gl_state (ctx->source_material,&options);
+  _cogl_flush_matrix_stacks();
 
   while (path_start < ctx->path_nodes->len)
     {
