@@ -49,7 +49,8 @@ clutter_stage_egl_unrealize (ClutterActor *actor)
 
   CLUTTER_MARK();
 
-  CLUTTER_ACTOR_CLASS (clutter_stage_egl_parent_class)->unrealize (actor);
+  if (CLUTTER_ACTOR_CLASS (clutter_stage_egl_parent_class)->unrealize != NULL)
+    CLUTTER_ACTOR_CLASS (clutter_stage_egl_parent_class)->unrealize (actor);
 
   if (stage_egl->egl_surface)
     {
@@ -187,8 +188,7 @@ clutter_stage_egl_realize (ClutterActor *actor)
           CLUTTER_NOTE (GL, "Created EGL Context");
         }
 
-      CLUTTER_NOTE (BACKEND, "Marking stage as realized and setting context");
-      CLUTTER_ACTOR_SET_FLAGS (stage_egl, CLUTTER_ACTOR_REALIZED);
+      CLUTTER_NOTE (BACKEND, "Setting context");
 
       /* eglnative can have only one stage */
       status = eglMakeCurrent (backend_egl->edpy,
@@ -220,9 +220,9 @@ clutter_stage_egl_realize (ClutterActor *actor)
 
 static void
 clutter_stage_egl_get_preferred_width (ClutterActor *self,
-                                       ClutterUnit   for_height,
-                                       ClutterUnit  *min_width_p,
-                                       ClutterUnit  *natural_width_p)
+                                       gfloat        for_height,
+                                       gfloat       *min_width_p,
+                                       gfloat       *natural_width_p)
 {
   ClutterStageEGL *stage_egl = CLUTTER_STAGE_EGL (self);
 
@@ -235,9 +235,9 @@ clutter_stage_egl_get_preferred_width (ClutterActor *self,
 
 static void
 clutter_stage_egl_get_preferred_height (ClutterActor *self,
-                                        ClutterUnit   for_width,
-                                        ClutterUnit  *min_height_p,
-                                        ClutterUnit  *natural_height_p)
+                                        gfloat        for_width,
+                                        gfloat       *min_height_p,
+                                        gfloat       *natural_height_p)
 {
   ClutterStageEGL *stage_egl = CLUTTER_STAGE_EGL (self);
 
@@ -253,8 +253,6 @@ clutter_stage_egl_dispose (GObject *gobject)
 {
   ClutterStageEGL *stage_egl = CLUTTER_STAGE_EGL (gobject);
 
-  clutter_actor_unrealize (CLUTTER_ACTOR (stage_egl));
-
   G_OBJECT_CLASS (clutter_stage_egl_parent_class)->dispose (gobject);
 }
 
@@ -278,7 +276,7 @@ static void
 clutter_stage_egl_set_fullscreen (ClutterStageWindow *stage_window,
                                   gboolean            fullscreen)
 {
-  g_warning ("Stage of type `%s' do not support ClutterStage::set_fullscreen",
+  g_warning ("Stage of type '%s' do not support ClutterStage::set_fullscreen",
              G_OBJECT_TYPE_NAME (stage_window));
 }
 
